@@ -1,4 +1,4 @@
-# Copyright 2014, 2015 IBM Corp.
+# Copyright 2014, 2018 IBM Corp.
 #
 # All Rights Reserved.
 #
@@ -25,6 +25,9 @@ _SCHEMA_VER140 = 'V1_4_0'
 _SCHEMA_VER150 = 'V1_5_0'
 _SCHEMA_VER160 = 'V1_6_0'
 _SCHEMA_VER170 = 'V1_7_0'
+_SCHEMA_VER180 = 'V1_8_0'
+_SCHEMA_VER190 = 'V1_9_0'
+_SCHEMA_VER1100 = 'V1_10_0'
 _ATTR_SCHEMA_VER = 'schemaVersion'
 _ATTR_KSV = 'ksv'
 DEFAULT_SCHEMA_ATTR = {_ATTR_SCHEMA_VER: _DEFAULT_SCHEMA_VERSION}
@@ -34,6 +37,9 @@ ATTR_KSV140 = {_ATTR_KSV: _SCHEMA_VER140}
 ATTR_KSV150 = {_ATTR_KSV: _SCHEMA_VER150}
 ATTR_KSV160 = {_ATTR_KSV: _SCHEMA_VER160}
 ATTR_KSV170 = {_ATTR_KSV: _SCHEMA_VER170}
+ATTR_KSV180 = {_ATTR_KSV: _SCHEMA_VER180}
+ATTR_KSV190 = {_ATTR_KSV: _SCHEMA_VER190}
+ATTR_KSV1100 = {_ATTR_KSV: _SCHEMA_VER1100}
 ATTR_SCHEMA_KSV130 = {_ATTR_KSV: _SCHEMA_VER130,
                       _ATTR_SCHEMA_VER: _SCHEMA_VER130}
 
