@@ -1,4 +1,4 @@
-# Copyright 2015 IBM Corp.
+# Copyright 2015, 2018 IBM Corp.
 #
 # All Rights Reserved.
 #
@@ -42,6 +42,7 @@ PPT_RATIO = 'ppt_ratio'
 # enumeration of accepted values starting from 0.
 ALLOWED_PPT_RATIOS = {'1:64': 0, '1:128': 1, '1:256': 2, '1:512': 3,
                       '1:1024': 4, '1:2048': 5, '1:4096': 6}
+ENFORCE_AFFINITY_CHECK = 'enforce_affinity_check'
 
 DED_PROCS = 'dedicated_proc'
 VCPU = 'vcpu'
@@ -61,6 +62,7 @@ AVAIL_PRIORITY = 'avail_priority'
 SRR_CAPABLE = 'srr_capability'
 PROC_COMPAT = 'processor_compatibility'
 ENABLE_LPAR_METRIC = 'enable_lpar_metric'
+SECURE_BOOT = 'secure_boot'
 
 # I/O configuration
 # Sure would love to change this to MAX_VIRT_IO_SLOTS or similar, but compat...
@@ -91,6 +93,7 @@ DEF_SPP = 0
 DEF_AVAIL_PRI = 127
 DEF_SRR = 'false'
 DEF_LPAR_METRIC = False
+DEF_SECURE_BOOT = 0
 DEF_PHYS_IO_SLOTS = None
 
 LOG = logging.getLogger(__name__)
@@ -184,7 +187,8 @@ class DefaultStandardize(Standardize):
                  uncapped_weight=DEF_UNCAPPED_WT, spp=DEF_SPP,
                  avail_priority=DEF_AVAIL_PRI, srr=DEF_SRR,
                  proc_compat=bp.LPARCompat.DEFAULT,
-                 enable_lpar_metric=DEF_LPAR_METRIC):
+                 enable_lpar_metric=DEF_LPAR_METRIC,
+                 secure_boot=DEF_SECURE_BOOT):
         """Initialize the standardizer
 
         :param mngd_sys: managed_system wrapper of the host to deploy to.
@@ -202,6 +206,7 @@ class DefaultStandardize(Standardize):
         :param proc_compat: processor compatibility mode value
         :param enable_lpar_metric: LPAR performance data collection attribute
             enabled only if value is true
+        :param secure_boot: The secure boot policy value
         """
 
         super(DefaultStandardize, self).__init__()
@@ -218,6 +223,7 @@ class DefaultStandardize(Standardize):
         self.srr = srr
         self.enable_lpar_metric = enable_lpar_metric
         self.proc_compat = proc_compat
+        self.secure_boot = secure_boot
 
     def _set_prop(self, attr, prop, base_prop, convert_func=str):
         """Copies a property if present or copies the base property."""
@@ -250,11 +256,32 @@ class DefaultStandardize(Standardize):
         ProcCompatMode(attrs.get(PROC_COMPAT),
                        host_modes=self.mngd_sys.proc_compat_modes,
                        allow_none=partial).validate()
+        secure_boot_cap = self._can_secure_boot_for_lpar(attrs.get(ENV, ''))
+        SecureBoot(attrs.get(SECURE_BOOT, DEF_SECURE_BOOT),
+                   secure_boot_cap).validate()
 
         # Validate fields specific to IBMi
         if attrs.get(ENV, '') == bp.LPARType.OS400:
             RestrictedIO(attrs.get(RESTRICTED_IO), allow_none=True).validate()
 
+        # Validate affinity check attribute based on host capability
+        host_affinity_cap = self.mngd_sys.get_capability(
+            'affinity_check_capable')
+        EnforceAffinityCheck(attrs.get(ENFORCE_AFFINITY_CHECK),
+                             host_affinity_cap).validate()
+
+    def _can_secure_boot_for_lpar(self, lpar_type):
+        if lpar_type == bp.LPARType.OS400:
+            # NOTE(edmondsw) Secure boot is initially supported only for RPA
+            # partitions (AIX/Linux). It is blocked at the schema level for
+            # IBMi. See https://bugs.launchpad.net/pypowervm/+bug/1805610
+            # TODO(edmondsw): If/when secure boot is supported for IBMi, a new
+            # capability will need to be checked here.
+            return False
+        else:
+            return self.mngd_sys.get_capability(
+                'partition_secure_boot_capable')
+
     def _validate_memory(self, attrs=None, partial=False):
         if attrs is None:
             attrs = self.attr
@@ -315,7 +342,10 @@ class DefaultStandardize(Standardize):
                           convert_func=SimplifiedRemoteRestart.convert_value)
         self._set_val(bld_attr, PROC_COMPAT, bp.LPARCompat.DEFAULT,
                       convert_func=ProcCompatMode.convert_value)
-
+        # See if the host is capable of secure boot before setting it.
+        if self._can_secure_boot_for_lpar(bld_attr[ENV]):
+            self._set_val(bld_attr, SECURE_BOOT, self.secure_boot,
+                          convert_func=int)
         # Build IBMi attributes
         if bld_attr[ENV] == bp.LPARType.OS400:
             self._set_val(bld_attr, CONSOLE, value='HMC')
@@ -706,6 +736,28 @@ class ProcCompatMode(ChoiceField):
             self._choices = host_modes
 
 
+class SecureBoot(IntBoundField):
+    """Secure boot policy.
+
+    """
+    _min_bound = 0
+    _max_bound = 9
+    _name = 'Secure Boot'
+
+    def __init__(self, value, host_cap, allow_none=False):
+        super(SecureBoot, self).__init__(value, allow_none=allow_none)
+        self.host_cap = host_cap
+
+    def validate(self):
+        """Performs the additional host capability check for secure boot."""
+        super(SecureBoot, self).validate()
+
+        if int(self.value) > 0 and not self.host_cap:
+            msg = _("The managed system or partition type does not support "
+                    "secure boot.")
+            raise ValueError(msg)
+
+
 class DedProcShareMode(ChoiceField):
     _choices = bp.DedicatedSharingMode.ALL_VALUES
     _name = 'Dedicated Processor Sharing Mode'
@@ -773,8 +825,27 @@ class PhysicalPageTableRatio(ChoiceField):
 
         # Validate the host capability
         if not self.host_cap and self.value:
-            msg = ("The managed system does not support setting the physical "
-                   "page table ratio.")
+            msg = _("The managed system does not support setting the physical "
+                    "page table ratio.")
+            raise ValueError(msg)
+
+
+class EnforceAffinityCheck(BoolField):
+    _name = 'Enforce Affinity Check'
+
+    def __init__(self, value, host_cap, allow_none=True):
+        super(EnforceAffinityCheck, self).__init__(
+            value, allow_none=allow_none)
+        self.host_cap = host_cap
+
+    def validate(self):
+        """Performs validation of the affinity check attribute."""
+        super(EnforceAffinityCheck, self).validate()
+
+        # Validate the host capability
+        if (str(self.value).lower() == 'true') and (not self.host_cap):
+            msg = _("The managed system does not support affinity score "
+                    "checks as part of migration.")
             raise ValueError(msg)
 
 
@@ -930,6 +1001,8 @@ class LPARBuilder(object):
         lpar_w.avail_priority = std[AVAIL_PRIORITY]
         lpar_w.proc_compat_mode = std[PROC_COMPAT]
         lpar_w.allow_perf_data_collection = std[ENABLE_LPAR_METRIC]
+        if std.get(SECURE_BOOT) is not None:
+            lpar_w.pending_secure_boot = std[SECURE_BOOT]
         # Host may not be capable of SRR, so only add it if it's in the
         # standardized attributes
         if std.get(SRR_CAPABLE) is not None:
