@@ -1,4 +1,4 @@
-# Copyright 2015 IBM Corp.
+# Copyright 2015, 2017 IBM Corp.
 #
 # All Rights Reserved.
 #
@@ -56,14 +56,22 @@ class LPARWrapperValidator(object):
         self.host_w = host_w
         self.cur_lpar_w = cur_lpar_w
 
-    def validate_all(self):
-        """Invoke attribute validation classes to perform validation"""
+    def validate_all(self, check_dlpar=True):
+        """Invoke attribute validation classes to perform validation
+
+        :param check_dlpar: indicates the need to validate the dlpar
+            capability. It is False when update is requested with force
+            option.
+        """
         ProcValidator(self.lpar_w, self.host_w,
-                      cur_lpar_w=self.cur_lpar_w).validate()
+                      cur_lpar_w=self.cur_lpar_w).validate(
+                          check_dlpar=check_dlpar)
         MemValidator(self.lpar_w, self.host_w,
-                     cur_lpar_w=self.cur_lpar_w).validate()
+                     cur_lpar_w=self.cur_lpar_w).validate(
+                         check_dlpar=check_dlpar)
         CapabilitiesValidator(self.lpar_w, self.host_w,
-                              cur_lpar_w=self.cur_lpar_w).validate()
+                              cur_lpar_w=self.cur_lpar_w).validate(
+                                  check_dlpar=check_dlpar)
 
 
 @six.add_metaclass(abc.ABCMeta)
@@ -85,15 +93,20 @@ class BaseValidator(object):
         self.host_w = host_w
         self.cur_lpar_w = cur_lpar_w
 
-    def validate(self):
-        """Determines what validation is requested and invokes it."""
+    def validate(self, check_dlpar=True):
+        """Determines what validation is requested and invokes it.
+
+        :param check_dlpar: flag indicating if we need to validate dlpar
+            capability.
+        """
         # Deploy
         self._populate_new_values()
         if self.cur_lpar_w is None:
             self._validate_deploy()
         # Resize
         else:
-            self._can_modify()
+            if check_dlpar:
+                self._can_modify()
             self._populate_resize_diffs()
             # Inactive Resize
             if self.cur_lpar_w.state == bp.LPARState.NOT_ACTIVATED:
@@ -176,8 +189,14 @@ class MemValidator(BaseValidator):
     :attr max_mem: maximum memory of the new lpar
     :attr min_mem: minimum memory of the new lpar
     :attr avail_mem: available memory on the host
+    :attr ppt_ratio: desired ppt ratio of the new lpar
     :attr res_name: name of the resource
     """
+    def __init__(self, lpar_w, host_w, cur_lpar_w=None):
+        super(MemValidator, self).__init__(lpar_w, host_w,
+                                           cur_lpar_w=cur_lpar_w)
+        self.ppt_ratio = None
+
     def _populate_new_values(self):
         """Set newly desired LPAR attributes as instance attributes."""
         mem_cfg = self.lpar_w.mem_config
@@ -186,6 +205,7 @@ class MemValidator(BaseValidator):
         self.min_mem = mem_cfg.min
         self.exp_fact = mem_cfg.exp_factor
         self.avail_mem = self.host_w.memory_free
+        self.ppt_ratio = mem_cfg.ppt_ratio
         self.res_name = _('memory')
 
     def _populate_resize_diffs(self):
@@ -216,6 +236,12 @@ class MemValidator(BaseValidator):
                      "the expansion factor. Power off virtual machine %s and "
                      "try again.") % self.cur_lpar_w.name)
             raise ValidatorException(msg)
+        if (self.ppt_ratio is not None and
+                self.ppt_ratio != curr_mem_cfg.ppt_ratio):
+            msg = ("The virtual machine must be powered off before changing "
+                   "the physical page table ratio. Power off virtual "
+                   "machine %s and try again.") % self.cur_lpar_w.name
+            raise ValidatorException(msg)
         # Common validations for both active & inactive resizes.
         self._validate_resize_common()
 
