@@ -1,4 +1,4 @@
-# Copyright 2014, 2017 IBM Corp.
+# Copyright 2014, 2018 IBM Corp.
 #
 # All Rights Reserved.
 #
@@ -35,6 +35,8 @@ _HOST_IP_ADDRESS = _PRIMARY_IP_ADDRESS
 _STATE = 'State'
 _SYSTEM_NAME = 'SystemName'
 _MASTER_MODE = 'IsPowerVMManagementMaster'
+_PROC_THROTTLE = 'ProcessorThrottling'
+_METER_POOL_ID = 'MeteredPoolID'
 
 _SYS_CAPABILITIES = 'AssociatedSystemCapabilities'
 _ACTIVE_LPM_CAP = u.xpath(
@@ -63,8 +65,29 @@ _DYN_SRR_CAP = u.xpath(
 _IBMi_NATIVE_IO_CAP = u.xpath(_SYS_CAPABILITIES, 'IBMiNativeIOCapable')
 _DISABLE_SECURE_BOOT_CAP = u.xpath(
     _SYS_CAPABILITIES, 'DisableSecureBootCapable')
+_PARTITION_SECURE_BOOT_CAP = u.xpath(
+    _SYS_CAPABILITIES, 'PartitionSecureBootCapable')
 _IOSLOT_OWNER_ASSMT_CAP = u.xpath(
     _SYS_CAPABILITIES, 'IOSlotOwnerAssignmentCapable')
+_DED_PROC_POOL_CAP = u.xpath(
+    _SYS_CAPABILITIES, 'DedicatedProcessorPartitionCapable')
+
+# Migration Constants
+_SYS_PROC_CONFIG = 'AssociatedSystemProcessorConfiguration'
+_PROC_COMPAT_MODES = u.xpath(
+    _SYS_PROC_CONFIG, 'SupportedPartitionProcessorCompatibilityModes')
+_MIN_PROC_UNITS_PER_CPU = u.xpath(
+    _SYS_PROC_CONFIG, 'MinimumProcessorUnitsPerVirtualProcessor')
+_MIGR_INFO = 'SystemMigrationInformation'
+_MAX_ACTIVE_MIGR = u.xpath(_MIGR_INFO, 'MaximumActiveMigrations')
+_MAX_INACTIVE_MIGR = u.xpath(_MIGR_INFO, 'MaximumInactiveMigrations')
+_ACTIVE_MIGR_RUNNING = u.xpath(
+    _MIGR_INFO, 'NumberOfActiveMigrationsInProgress')
+_INACTIVE_MIGR_RUNNING = u.xpath(
+    _MIGR_INFO, 'NumberOfInactiveMigrationsInProgress')
+_MAX_FIRMWARE_MIGR = u.xpath(_MIGR_INFO, 'MaximumFirmwareActiveMigrations')
+_AFFINITY_CHECK_CAP = u.xpath(
+    _MIGR_INFO, 'LogicalPartitionAffinityCheckCapable')
 
 _CAPABILITY_MAP = {
     'active_lpar_mobility_capable': {
@@ -82,6 +105,8 @@ _CAPABILITY_MAP = {
         'prop': _SIMP_REMOTE_RESTART_CAP, 'default': False},
     'physical_page_table_ratio_capable': {
         'prop': _PPT_CAP, 'default': False},
+    'affinity_check_capable': {
+        'prop': _AFFINITY_CHECK_CAP, 'default': False},
     'active_memory_expansion_capable': {
         'prop': _AME_CAP, 'default': False},
     # aix_capable defaults to True for backward compat (that is what we
@@ -104,8 +129,12 @@ _CAPABILITY_MAP = {
         'prop': _IBMi_NATIVE_IO_CAP, 'default': False},
     'disable_secure_boot_capable': {
         'prop': _DISABLE_SECURE_BOOT_CAP, 'default': False},
+    'partition_secure_boot_capable': {
+        'prop': _PARTITION_SECURE_BOOT_CAP, 'default': False},
     'ioslot_owner_assignment_capable': {
         'prop': _IOSLOT_OWNER_ASSMT_CAP, 'default': False},
+    'dedicated_processor_partition_capable': {
+        'prop': _DED_PROC_POOL_CAP, 'default': True},
 }
 
 _SYS_MEM_CONFIG = 'AssociatedSystemMemoryConfiguration'
@@ -117,21 +146,6 @@ _SYS_FIRMWARE_MEM = u.xpath(_SYS_MEM_CONFIG, 'MemoryUsedByHypervisor')
 _PAGE_TABLE_RATIO = u.xpath(_SYS_MEM_CONFIG, 'DefaultHardwarePageTableRatio')
 _DEFAULT_PPT_RATIO = u.xpath(_SYS_MEM_CONFIG, 'DefaultPhysicalPageTableRatio')
 
-# Migration Constants
-_SYS_PROC_CONFIG = 'AssociatedSystemProcessorConfiguration'
-_PROC_COMPAT_MODES = u.xpath(
-    _SYS_PROC_CONFIG, 'SupportedPartitionProcessorCompatibilityModes')
-_MIN_PROC_UNITS_PER_CPU = u.xpath(
-    _SYS_PROC_CONFIG, 'MinimumProcessorUnitsPerVirtualProcessor')
-_MIGR_INFO = 'SystemMigrationInformation'
-_MAX_ACTIVE_MIGR = u.xpath(_MIGR_INFO, 'MaximumActiveMigrations')
-_MAX_INACTIVE_MIGR = u.xpath(_MIGR_INFO, 'MaximumInactiveMigrations')
-_ACTIVE_MIGR_RUNNING = u.xpath(
-    _MIGR_INFO, 'NumberOfActiveMigrationsInProgress')
-_INACTIVE_MIGR_RUNNING = u.xpath(
-    _MIGR_INFO, 'NumberOfInactiveMigrationsInProgress')
-_MAX_FIRMWARE_MIGR = u.xpath(_MIGR_INFO, 'MaximumFirmwareActiveMigrations')
-
 _PROC_UNITS_INSTALLED = u.xpath(
     _SYS_PROC_CONFIG, 'InstalledSystemProcessorUnits')
 
@@ -387,6 +401,14 @@ class System(ewrap.EntryWrapper):
         """
         return self._get_val_bool(_MASTER_MODE, True)
 
+    @property
+    def metered_pool_id(self):
+        return self._get_val_str(_METER_POOL_ID)
+
+    @property
+    def processor_is_throttled(self):
+        return self._get_val_bool(_PROC_THROTTLE)
+
 
 @ewrap.ElementWrapper.pvm_type(_ASIO_ROOT, has_metadata=True)
 class ASIOConfig(ewrap.ElementWrapper):
