@@ -1,4 +1,4 @@
-# Copyright 2014, 2017 IBM Corp.
+# Copyright 2014, 2018 IBM Corp.
 #
 # All Rights Reserved.
 #
@@ -23,6 +23,7 @@ from oslo_log import log as logging
 import six
 
 import pypowervm.const as c
+import pypowervm.entities as ent
 import pypowervm.exceptions as ex
 from pypowervm.i18n import _
 import pypowervm.util as u
@@ -32,9 +33,41 @@ LOG = logging.getLogger(__name__)
 
 UDID = 'UniqueDeviceID'
 
-# Storage QoS Constants - common to Physical Volume & Virtual Disk
-_READ_IOPS = 'ReadIOPS'
-_WRITE_IOPS = 'WriteIOPS'
+# The following are common to all VSCSI storage types
+# Storage QoS Constants
+_STOR_READ_IOPS = 'ReadIOPS'
+_STOR_WRITE_IOPS = 'WriteIOPS'
+# Storage Encryption Constants
+_STOR_ENCRYPTION_STATE = 'EncryptionState'
+_STOR_ENCRYPTION_KEY = 'EncryptionKey'
+_STOR_ENCRYPTION_AGENT = 'EncryptionAgent'
+# Device tag
+_STOR_TAG = 'Tag'
+# Emulate model alias
+_STOR_EMULATE_MODEL = 'EmulateModel'
+
+_STOR_EL_ORDER = (_STOR_READ_IOPS, _STOR_WRITE_IOPS, _STOR_ENCRYPTION_STATE,
+                  _STOR_ENCRYPTION_KEY, _STOR_ENCRYPTION_AGENT, _STOR_TAG,
+                  _STOR_EMULATE_MODEL)
+
+
+class _EncryptionState(object):
+    """From EncryptionState.Enum.
+
+    This class is part of an experimental API change and may be subject to
+    breaking changes until it is publicized.
+    """
+    UNENCRYPTED = 'Unencrypted'
+    FORMATTED = 'Formatted'
+    UNLOCKED = 'Unlocked'
+
+
+# LUKS-specific encryptor constants
+_LUKS_ENCRYPTOR = 'LUKSEncryptor'
+_LUKS_CIPHER = 'Cipher'
+_LUKS_KEY_SIZE = 'KeySize'
+_LUKS_HASH_SPEC = 'Hash'
+_LUKS_EL_ORDER = [_LUKS_CIPHER, _LUKS_KEY_SIZE, _LUKS_HASH_SPEC]
 
 # "Any" server adapters are SCSI adapters without client
 # adapters that map to remote LPAR slot number 65535. They
@@ -44,8 +77,6 @@ ANY_SLOT = 65535
 
 # Virtual Disk Constants
 DISK_ROOT = 'VirtualDisk'
-_DISK_READ_IOPS = _READ_IOPS
-_DISK_WRITE_IOPS = _WRITE_IOPS
 _DISK_CAPACITY = 'DiskCapacity'
 _DISK_LABEL = 'DiskLabel'
 DISK_NAME = 'DiskName'
@@ -57,12 +88,13 @@ _DISK_UDID = UDID
 _DISK_TYPE = 'VirtualDiskType'
 _DISK_BACKSTORE_TYPE = 'BackStoreType'
 _DISK_FILEFORMAT = 'FileFormat'
+_DISK_RBD_USER = 'RbdUser'
 _DISK_OPTIONAL_PARMS = 'OptionalParameters'
-_VDISK_EL_ORDER = [_DISK_READ_IOPS, _DISK_WRITE_IOPS, _DISK_CAPACITY,
-                   _DISK_LABEL, DISK_NAME, _DISK_MAX_LOGICAL_VOLS,
-                   _DISK_PART_SIZE, _DISK_VG, _DISK_BASE, _DISK_UDID,
-                   _DISK_TYPE, _DISK_BACKSTORE_TYPE, _DISK_FILEFORMAT,
-                   _DISK_OPTIONAL_PARMS]
+_VDISK_EL_ORDER = _STOR_EL_ORDER + (
+    _DISK_CAPACITY, _DISK_LABEL, DISK_NAME, _DISK_MAX_LOGICAL_VOLS,
+    _DISK_PART_SIZE, _DISK_VG, _DISK_BASE, _DISK_UDID, _DISK_TYPE,
+    _DISK_BACKSTORE_TYPE, _DISK_FILEFORMAT, _DISK_RBD_USER,
+    _DISK_OPTIONAL_PARMS)
 
 
 class VDiskType(object):
@@ -73,25 +105,35 @@ class VDiskType(object):
 
 
 class BackStoreType(object):
-    """From BackStoreType.Enum."""
+    """From BackStoreType.Enum
+
+    Desribes the type of backstore handler to use for VDisks.
+    FILE_IO, USER_QCOW, and LOOP are used with the FileIO VDisk type.
+    USER_RBD is used with the RBD VDisk type.
+    """
     # A kernel-space handler that supports raw files.
     FILE_IO = 'fileio'
     # A user-space handler that supports RAW, QCOW or QCOW2 files.
     USER_QCOW = 'user:qcow'
-    # A user-space handler that supports rbd
+    # Create a loop device for the file, and use the kernel-space block
+    # handler. LOOP has higher performance than FILE_IO.
+    LOOP = 'loop'
+    # A user-space handler that supports rbd. (Used with RBD)
     USER_RBD = 'user:rbd'
 
 
 class FileFormatType(object):
-    """From FileFormatType.Enum"""
+    """From FileFormatType.Enum
+
+    The format type of the image that will be stored in the VDisk (aka LV).
+    """
     RAW = 'raw'
     QCOW2 = 'qcow2'
 
+
 # Physical Volume Constants
 PVS = 'PhysicalVolumes'
 PHYS_VOL = 'PhysicalVolume'
-_PV_READ_IOPS = _READ_IOPS
-_PV_WRITE_IOPS = _WRITE_IOPS
 _PV_AVAIL_PHYS_PART = 'AvailablePhysicalPartitions'
 _PV_VOL_DESC = 'Description'
 _PV_LOC_CODE = 'LocationCode'
@@ -108,12 +150,11 @@ _PV_VOL_UNIQUE_ID = 'VolumeUniqueID'
 _PV_FC_BACKED = 'IsFibreChannelBacked'
 _PV_STG_LABEL = 'StorageLabel'
 _PV_PG83 = 'DescriptorPage83'
-_PV_EL_ORDER = [_PV_READ_IOPS, _PV_WRITE_IOPS, _PV_AVAIL_PHYS_PART,
-                _PV_VOL_DESC, _PV_LOC_CODE, _PV_PERSISTENT_RESERVE,
-                _PV_RES_POLICY, _PV_RES_POLICY_ALGO, _PV_TOTAL_PHYS_PARTS,
-                _PV_UDID, _PV_AVAIL_FOR_USE, _PV_VOL_SIZE, _PV_VOL_NAME,
-                _PV_VOL_STATE, _PV_VOL_UNIQUE_ID, _PV_FC_BACKED,
-                _PV_STG_LABEL, _PV_PG83]
+_PV_EL_ORDER = _STOR_EL_ORDER + (
+    _PV_AVAIL_PHYS_PART, _PV_VOL_DESC, _PV_LOC_CODE, _PV_PERSISTENT_RESERVE,
+    _PV_RES_POLICY, _PV_RES_POLICY_ALGO, _PV_TOTAL_PHYS_PARTS, _PV_UDID,
+    _PV_AVAIL_FOR_USE, _PV_VOL_SIZE, _PV_VOL_NAME, _PV_VOL_STATE,
+    _PV_VOL_UNIQUE_ID, _PV_FC_BACKED, _PV_STG_LABEL, _PV_PG83)
 
 
 class PVState(object):
@@ -122,13 +163,15 @@ class PVState(object):
     REMOVED = "removed"
     VARIED_OFF = "varied off"
 
+
 # Virtual Optical Media Constants
 VOPT_ROOT = 'VirtualOpticalMedia'
 VOPT_NAME = 'MediaName'
-_VOPT_SIZE = 'Size'
 _VOPT_UDID = 'MediaUDID'
 _VOPT_MOUNT_TYPE = 'MountType'
-_VOPT_EL_ORDER = [VOPT_NAME, _VOPT_UDID, _VOPT_MOUNT_TYPE, _VOPT_SIZE]
+_VOPT_SIZE = 'Size'
+_VOPT_EL_ORDER = _STOR_EL_ORDER + (
+    VOPT_NAME, _VOPT_UDID, _VOPT_MOUNT_TYPE, _VOPT_SIZE)
 
 # Virtual Media Repository Constants
 _VREPO_ROOT = 'VirtualMediaRepository'
@@ -164,8 +207,10 @@ _LU_TYPE = 'LogicalUnitType'
 _LU_CLONED_FROM = 'ClonedFrom'
 _LU_IN_USE = 'InUse'
 _LU_NAME = 'UnitName'
-_LU_EL_ORDER = (_LU_THIN, _LU_UDID, _LU_CAPACITY, _LU_TYPE, _LU_CLONED_FROM,
-                _LU_IN_USE, _LU_NAME)
+_LU_MIG = 'LogicalUnitMigration'
+_LU_EL_ORDER = _STOR_EL_ORDER + (
+    _LU_THIN, _LU_UDID, _LU_CAPACITY, _LU_TYPE, _LU_CLONED_FROM, _LU_IN_USE,
+    _LU_NAME, _LU_MIG)
 
 
 class LUType(object):
@@ -174,6 +219,7 @@ class LUType(object):
     IMAGE = "VirtualIO_Image"
     AMS = "VirtualIO_Active_Memory_Sharing"
 
+
 _CAPACITY = 'Capacity'
 
 # Tier Constants
@@ -579,32 +625,153 @@ class _StorageQoS(ewrap.Wrapper):
     @property
     def read_iops_limit(self):
         """The device's I/O Read limit"""
-        return self._get_val_int(_READ_IOPS)
+        return self._get_val_int(_STOR_READ_IOPS)
 
     @read_iops_limit.setter
     def read_iops_limit(self, new_read_iops_limit):
         self.set_parm_value(
-            _READ_IOPS, new_read_iops_limit, attrib=c.ATTR_KSV170)
+            _STOR_READ_IOPS, new_read_iops_limit, attrib=c.ATTR_KSV170)
 
     @property
     def write_iops_limit(self):
         """The device's I/O Write limit"""
-        return self._get_val_int(_WRITE_IOPS)
+        return self._get_val_int(_STOR_WRITE_IOPS)
 
     @write_iops_limit.setter
     def write_iops_limit(self, new_write_iops_limit):
         self.set_parm_value(
-            _WRITE_IOPS, new_write_iops_limit, attrib=c.ATTR_KSV170)
+            _STOR_WRITE_IOPS, new_write_iops_limit, attrib=c.ATTR_KSV170)
+
+
+@ewrap.ElementWrapper.pvm_type(_LUKS_ENCRYPTOR, has_metadata=True,
+                               child_order=_LUKS_EL_ORDER)
+class _LUKSEncryptor(ewrap.ElementWrapper):
+    """An encryption agent that uses Linux Unified Key Setup (LUKS).
+
+    This class is part of an experimental API change and may be subject to
+    breaking changes until it is publicized.
+    """
+
+    @classmethod
+    def bld(cls, adapter, cipher=None, key_size=None, hash_spec=None):
+        """Creates a new LUKSEncryptor wrapper.
+
+        This can be attached to a disk wrapper during disk create and update
+        operations to specify encryption parameters for the device.
+
+        :param adapter: A pypowervm.adapter.Adapter (for traits, etc.)
+        :param cipher: A string containing the encryption algorithm, mode, and
+                       initialization vector (e.g. aes-xts-plain64). Optional.
+        :param key_size: The key size of the data encryption key. Optional.
+        :param hash_spec: The hash algorithm used for data encryption key
+                          derivation. Optional.
+        """
+        encryptor = super(_LUKSEncryptor, cls)._bld(adapter)
+        if cipher is not None:
+            encryptor.cipher = cipher
+        if key_size is not None:
+            encryptor.key_size = key_size
+        if hash_spec is not None:
+            encryptor.hash_spec = hash_spec
+        return encryptor
+
+    @property
+    def cipher(self):
+        """Cipher mode for translating between ciphertext and cleartext."""
+        return self._get_val_str(_LUKS_CIPHER)
+
+    @cipher.setter
+    def cipher(self, new_cipher):
+        self.set_parm_value(_LUKS_CIPHER, new_cipher, attrib=c.ATTR_KSV170)
+
+    @property
+    def key_size(self):
+        """Size of the master encryption key used for data encryption."""
+        return self._get_val_int(_LUKS_KEY_SIZE)
+
+    @key_size.setter
+    def key_size(self, new_key_size):
+        self.set_parm_value(_LUKS_KEY_SIZE, new_key_size, attrib=c.ATTR_KSV170)
+
+    @property
+    def hash_spec(self):
+        """Hash algorithm used for key derivation."""
+        return self._get_val_str(_LUKS_HASH_SPEC)
+
+    @hash_spec.setter
+    def hash_spec(self, new_hash_spec):
+        self.set_parm_value(_LUKS_HASH_SPEC, new_hash_spec,
+                            attrib=c.ATTR_KSV170)
+
+
+@ewrap.Wrapper.base_pvm_type
+class _StorageEncryption(ewrap.Wrapper):
+    """Encryption properties/methods common to PV and VDisk.
+
+    The members of this class are part of an experimental API change and may be
+    subject to breaking changes until they are publicized.
+    """
+
+    @property
+    def _encryption_state(self):
+        """The device's encryption state
+
+        This property is part of an experimental API change and may be subject
+        to breaking changes until it is publicized.
+        """
+        return self._get_val_str(_STOR_ENCRYPTION_STATE)
+
+    @_encryption_state.setter
+    def _encryption_state(self, new_encryption_state):
+        self.set_parm_value(
+            _STOR_ENCRYPTION_STATE, new_encryption_state, attrib=c.ATTR_KSV170)
+
+    @property
+    def _encryption_agent(self):
+        """The encryption agent used to encrypt the device.
+
+        This property is part of an experimental API change and may be subject
+        to breaking changes until it is publicized.
+        """
+        elem = self._find(_STOR_ENCRYPTION_AGENT)
+        if elem is None:
+            return None
+        agent_elems = list(elem)
+        if len(agent_elems) != 1:
+            return None
+        return ewrap.ElementWrapper.wrap(agent_elems[0])
+
+    @_encryption_agent.setter
+    def _encryption_agent(self, new_encryption_agent):
+        agent_elem = ent.Element(_STOR_ENCRYPTION_AGENT, self.adapter,
+                                 attrib=c.ATTR_KSV170)
+        if new_encryption_agent is not None:
+            agent_elem.inject(new_encryption_agent.element)
+        self.inject(agent_elem)
+
+    @property
+    def _encryption_key(self):
+        """The encryption key used to format and unlock an encrypted device
+
+        This property is part of an experimental API change and may be subject
+        to breaking changes until it is publicized.
+        """
+        return self._get_val_str(_STOR_ENCRYPTION_KEY)
+
+    @_encryption_key.setter
+    def _encryption_key(self, new_encryption_key):
+        self.set_parm_value(
+            _STOR_ENCRYPTION_KEY, new_encryption_key, attrib=c.ATTR_KSV170)
 
 
 @ewrap.ElementWrapper.pvm_type(PHYS_VOL, has_metadata=True,
                                child_order=_PV_EL_ORDER)
-class PV(ewrap.ElementWrapper, _StorageQoS):
+class PV(ewrap.ElementWrapper, _StorageQoS, _StorageEncryption):
     """A physical volume that backs a Volume Group."""
     target_dev_type = PVTargetDev
 
     @classmethod
-    def bld(cls, adapter, name, udid=None):
+    def bld(cls, adapter, name, udid=None, tag=None, emulate_model=None):
         """Creates the a fresh PV wrapper.
 
         This should be used when wishing to add physical volumes to a Volume
@@ -617,13 +784,21 @@ class PV(ewrap.ElementWrapper, _StorageQoS):
         :param name: The name of the physical volume on the Virtual I/O Server
                      to add to the Volume Group.  Ex. 'hdisk1'.
         :param udid: Universal Disk Identifier.
-        :returns: An Element that can be used for a PhysicalVolume create.
+        :param tag: String with which to tag the physical device upon mapping.
+        :param emulate_model: Boolean emulate model alias flag to set on the
+                              physical device upon mapping.
+        :returns: An Element that can be used for a PhysicalVolume create or
+                  mapping.
         """
         pv = super(PV, cls)._bld(adapter)
         # Assignment order is significant
         if udid:
             pv.udid = udid
         pv.name = name
+        if tag:
+            pv.tag = tag
+        if emulate_model is not None:
+            pv.emulate_model = emulate_model
         return pv
 
     @property
@@ -713,10 +888,37 @@ class PV(ewrap.ElementWrapper, _StorageQoS):
                         {'pg83_raw': encoded, 'type_error': te.args[0]})
         return None
 
+    @property
+    def tag(self):
+        return self._get_val_str(_STOR_TAG)
+
+    @tag.setter
+    def tag(self, tag):
+        """Set the tag on the storage element.
+
+        NOTE: This is only to be used when adding the storage element to a
+        VSCSI mapping.  It is ignored by .update().
+        """
+        self.set_parm_value(_STOR_TAG, tag, attrib=c.ATTR_KSV190)
+
+    @property
+    def emulate_model(self):
+        return self._get_val_bool(_STOR_EMULATE_MODEL, default=True)
+
+    @emulate_model.setter
+    def emulate_model(self, em):
+        """Set the emulate model alias flag on the storage element.
+
+        NOTE: This is only to be used when adding the storage element to a
+        VSCSI mapping.  It is ignored by .update().
+        """
+        self.set_parm_value(_STOR_EMULATE_MODEL, u.sanitize_bool_for_api(em),
+                            attrib=c.ATTR_KSV190)
+
 
 @ewrap.Wrapper.base_pvm_type
 class _VDisk(ewrap.ElementWrapper):
-    """Methods common to VDisk and FileIO."""
+    """Methods common to VDisk, FileIO, and RBD."""
 
     @property
     def name(self):
@@ -780,6 +982,33 @@ class _VDisk(ewrap.ElementWrapper):
         """
         self.set_parm_value(_DISK_FILEFORMAT, val, attrib=c.ATTR_KSV150)
 
+    @property
+    def tag(self):
+        return self._get_val_str(_STOR_TAG)
+
+    @tag.setter
+    def tag(self, tag):
+        """Set the tag on the storage element.
+
+        NOTE: This is only to be used when adding the storage element to a
+        VSCSI mapping.  It is ignored by .update().
+        """
+        self.set_parm_value(_STOR_TAG, tag, attrib=c.ATTR_KSV190)
+
+    @property
+    def emulate_model(self):
+        return self._get_val_bool(_STOR_EMULATE_MODEL, default=True)
+
+    @emulate_model.setter
+    def emulate_model(self, em):
+        """Set the emulate model alias flag on the storage element.
+
+        NOTE: This is only to be used when adding the storage element to a
+        VSCSI mapping.  It is ignored by .update().
+        """
+        self.set_parm_value(_STOR_EMULATE_MODEL, u.sanitize_bool_for_api(em),
+                            attrib=c.ATTR_KSV190)
+
 
 @ewrap.ElementWrapper.pvm_type(DISK_ROOT, has_metadata=True,
                                child_order=_VDISK_EL_ORDER)
@@ -792,11 +1021,17 @@ class FileIO(_VDisk):
     target_dev_type = VDiskTargetDev
 
     @classmethod
-    def bld_ref(cls, adapter, path, backstore_type=None):
+    def bld_ref(cls, adapter, path, backstore_type=None, tag=None,
+                emulate_model=None):
         """Creates a FileIO reference for inclusion in a VSCSIMapping.
 
         :param adapter: A pypowervm.adapter.Adapter for the REST API.
         :param path: The file system path of the File I/O object.
+        :param backstore_type: The type of backing storage, one of the
+                               BackStoreType enum values.
+        :param tag: String with which to tag the device upon mapping.
+        :param emulate_model: Boolean emulate model alias flag to set on the
+                              physical device upon mapping.
         :return: An Element that can be attached to a VSCSIMapping to create a
                  File I/O mapping on the server.
         """
@@ -806,6 +1041,10 @@ class FileIO(_VDisk):
         fio._vdtype(VDiskType.FILE)
         if backstore_type is not None:
             fio._backstore_type(backstore_type)
+        if tag:
+            fio.tag = tag
+        if emulate_model is not None:
+            fio.emulate_model = emulate_model
         return fio
 
     # Maintained for backward compatibility.  FileIOs aren't created by REST.
@@ -828,31 +1067,44 @@ class RBD(_VDisk):
     target_dev_type = VDiskTargetDev
 
     @classmethod
-    def bld_ref(cls, adapter, name):
-        """Creates a FileIO reference for inclusion in a VSCSIMapping.
+    def bld_ref(cls, adapter, name, tag=None, emulate_model=None, user=None):
+        """Creates a RBD reference for inclusion in a VSCSIMapping.
 
         :param adapter: A pypowervm.adapter.Adapter for the REST API.
-        :param path: The file system path of the File I/O object.
+        :param name: The name of the RBD object.  Also used as the label.
+        :param tag: String with which to tag the device upon mapping.
+        :param emulate_model: Boolean emulate model alias flag to set on the
+                              physical device upon mapping.
+        :param user: The user id used to access the rbd cluster.
         :return: An Element that can be attached to a VSCSIMapping to create a
-                 File I/O mapping on the server.
+                 RBD mapping on the server.
         """
         rbd = super(RBD, cls)._bld(adapter)
         rbd.name = name
         rbd._label(name)
         rbd._vdtype(VDiskType.RBD)
         rbd._backstore_type(BackStoreType.USER_RBD)
+        if tag:
+            rbd.tag = tag
+        if emulate_model is not None:
+            rbd.emulate_model = emulate_model
+        if user is not None:
+            rbd._user(user)
         return rbd
 
+    def _user(self, user):
+        self.set_parm_value(_DISK_RBD_USER, user, attrib=c.ATTR_KSV190)
+
 
 @ewrap.ElementWrapper.pvm_type(DISK_ROOT, has_metadata=True,
                                child_order=_VDISK_EL_ORDER)
-class VDisk(_VDisk, _StorageQoS):
-    """A virtual disk that can be attached to a VM."""
+class VDisk(_VDisk, _StorageQoS, _StorageEncryption):
+    """A Logical Volume virtual disk that can be attached to a VM."""
     target_dev_type = VDiskTargetDev
 
     @classmethod
     def bld(cls, adapter, name, capacity, label=None, base_image=None,
-            file_format=None):
+            file_format=None, tag=None, emulate_model=None):
         """Creates a VDisk Wrapper for creating a new VDisk.
 
         This should be used when the user wishes to add a new Virtual Disk to
@@ -869,6 +1121,9 @@ class VDisk(_VDisk, _StorageQoS):
                            Not required.
         :param file_format: (Optional) File format of VDisk.  See
                             FileFormatType enumeration for valid formats.
+        :param tag: String with which to tag the device upon mapping.
+        :param emulate_model: Boolean emulate model alias flag to set on the
+                              physical device upon mapping.
         :returns: An Element that can be used for a VirtualDisk create.
         """
         vd = super(VDisk, cls)._bld(adapter)
@@ -880,13 +1135,23 @@ class VDisk(_VDisk, _StorageQoS):
             vd._base_image(base_image)
         if file_format:
             vd._file_format(file_format)
+        vd._vdtype(VDiskType.LV)
+        if tag:
+            vd.tag = tag
+        if emulate_model is not None:
+            vd.emulate_model = emulate_model
         return vd
 
     @classmethod
-    def bld_ref(cls, adapter, name):
+    def bld_ref(cls, adapter, name, tag=None, emulate_model=None):
         """Creates a VDisk Wrapper for referring to an existing VDisk."""
         vd = super(VDisk, cls)._bld(adapter)
         vd.name = name
+        vd._vdtype(VDiskType.LV)
+        if tag:
+            vd.tag = tag
+        if emulate_model is not None:
+            vd.emulate_model = emulate_model
         return vd
 
     @property
@@ -894,6 +1159,10 @@ class VDisk(_VDisk, _StorageQoS):
         return self.get_href(_DISK_VG, one_result=True)
 
 
+# Alias for VDisk making it explicit that it's a Logical Volume type
+LV = VDisk
+
+
 @six.add_metaclass(abc.ABCMeta)
 @ewrap.Wrapper.base_pvm_type
 class _LUBase(ewrap.Wrapper):
@@ -906,7 +1175,8 @@ class _LUBase(ewrap.Wrapper):
     target_dev_type = LUTargetDev
 
     @classmethod
-    def bld(cls, adapter, name, capacity, thin=None, typ=None, clone=None):
+    def bld(cls, adapter, name, capacity, thin=None, typ=None, clone=None,
+            tag=None, emulate_model=None):
         """Build a fresh wrapper for LU creation within an SSP.
 
         :param adapter: A pypowervm.adapter.Adapter (for traits, etc.)
@@ -916,6 +1186,9 @@ class _LUBase(ewrap.Wrapper):
         :param typ: Logical Unit type, one of the LUType values.
         :param clone: If the new LU is to be a linked clone, this param is a
                       LU(Ent) wrapper representing the backing image LU.
+        :param tag: String with which to tag the device upon mapping.
+        :param emulate_model: Boolean emulate model alias flag to set on the
+                              physical device upon mapping.
         :return: A new LU wrapper suitable for adding to SSP.logical_units
                  prior to update.
         """
@@ -930,10 +1203,14 @@ class _LUBase(ewrap.Wrapper):
             lu._cloned_from_udid(clone.udid)
             # New LU must be at least as big as the backing LU.
             lu._capacity(max(capacity, clone.capacity))
+        if tag:
+            lu.tag = tag
+        if emulate_model is not None:
+            lu.emulate_model = emulate_model
         return lu
 
     @classmethod
-    def bld_ref(cls, adapter, name, udid):
+    def bld_ref(cls, adapter, name, udid, tag=None, emulate_model=None):
         """Creates the a fresh LU wrapper.
 
         The name matches the device name on the system.
@@ -941,11 +1218,18 @@ class _LUBase(ewrap.Wrapper):
         :param adapter: A pypowervm.adapter.Adapter (for traits, etc.)
         :param name: The name of the logical unit on the Virtual I/O Server.
         :param udid: Universal Disk Identifier.
+        :param tag: String with which to tag the device upon mapping.
+        :param emulate_model: Boolean emulate model alias flag to set on the
+                              physical device upon mapping.
         :returns: An Element that can be used for a PhysicalVolume create.
         """
         lu = super(_LUBase, cls)._bld(adapter)
         lu._name(name)
         lu._udid(udid)
+        if tag:
+            lu.tag = tag
+        if emulate_model is not None:
+            lu.emulate_model = emulate_model
         return lu
 
     def __eq__(self, other):
@@ -1020,6 +1304,33 @@ class _LUBase(ewrap.Wrapper):
     def in_use(self):
         return self._get_val_bool(_LU_IN_USE, default=None)
 
+    @property
+    def tag(self):
+        return self._get_val_str(_STOR_TAG)
+
+    @tag.setter
+    def tag(self, tag):
+        """Set the tag on the storage element.
+
+        NOTE: This is only to be used when adding the storage element to a
+        VSCSI mapping.  It is ignored by .update().
+        """
+        self.set_parm_value(_STOR_TAG, tag, attrib=c.ATTR_KSV190)
+
+    @property
+    def emulate_model(self):
+        return self._get_val_bool(_STOR_EMULATE_MODEL, default=True)
+
+    @emulate_model.setter
+    def emulate_model(self, em):
+        """Set the emulate model alias flag on the storage element.
+
+        NOTE: This is only to be used when adding the storage element to a
+        VSCSI mapping.  It is ignored by .update().
+        """
+        self.set_parm_value(_STOR_EMULATE_MODEL, u.sanitize_bool_for_api(em),
+                            attrib=c.ATTR_KSV190)
+
 
 @ewrap.ElementWrapper.pvm_type('LogicalUnit', has_metadata=True,
                                child_order=_LU_EL_ORDER)
